#! /bin/env python3
# encoding: utf-8

# --------------------------------------------------

cdef class ndict(dict):
	'''
	A dictionary superclass whose keys are exposed as attributes.
	
	>>> d = ndict()
	>>> d.alpha = 1
	>>> d['alpha']
	1
	>>> d['beta'] = 42
	>>> d.beta
	42
	>>> d
	{'alpha': 1, 'beta': 42}
	'''
	
	def __getattr__(self, str name):
		if name in self:
			return self[name]
		elif name.replace('_', '-') in self:
			return self[name.replace('_', '-')]
		else:
			raise AttributeError("'ndict' object has no attribute '%s'" %(name))
	
	def __setattr__(self, str name, value):
		self[name] = value

# --------------------------------------------------

class map:
	'''
	An ordered dict.
	'''
	
	def __init__(self, source=None):
		'''
		Initialize the map.
		
		The source is a sequence of (key, value) tuples.
		
		TODO fixme bug
		>>> map(((1, 2, 3, 4), ('a', 'b', 'c', 'd')))
		'''
		
		if source:
			self.keys, self.vals = zip(*source)
		else:
			self.keys = []
			self.vals = []
	
	def __get_index(self, key):
		if key in self.keys:
			return self.keys.index(key)
		else:
			return None
	
	def __getitem__(self, key):
		i = self.__get_index(key)
		
		if i is None:
			raise KeyError(key)
		else:
			return self.vals[i]
	
	def __setitem__(self, key, val):
		i = self.__get_index(key)
		
		if i is None:
			self.keys.append(key)
			self.vals.append(val)
		else:
			self.vals[i] = val
	
	def __contains__(self, item):
		return item in self.keys
	
	def index(self, item):
		return self.keys.index(item)
	
	def sort(self, key=None):
		tmp_keys = self.keys
		tmp_vals = self.vals
		
		self.keys = []
		self.vals = []
		
		for K, V in sorted(zip(tmp_keys, tmp_vals), key=key):
			self.keys.append(K)
			self.vals.append(V)
	
	@property
	def items(self):
		return zip(self.keys, self.vals)
	
	def __len__(self):
		return len(self.vals)
	
	def __repr__(self):
		pairs = []
		
		for i in range(len(self.keys)):
			pairs.append('%s: %s' %(repr(self.keys[i]), repr(self.vals[i])))
		
		return '<{%s}>' %(', '.join(pairs))
